==> ginkgo-1.6.0.tar.gz: ginkgo-1.6.0/benchmark/sparse_blas/operations.hpp <==
/*******************************<GINKGO LICENSE>******************************
Copyright (c) 2017-2023, the Ginkgo authors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************<GINKGO LICENSE>*******************************/

#include <ginkgo/ginkgo.hpp>


#include <tuple>


#include <rapidjson/document.h>


#include "benchmark/utils/types.hpp"


class BenchmarkOperation {
public:
    virtual ~BenchmarkOperation() = default;

    /**
     * Computes an estimate for the number of FLOPs executed by the operation.
     */
    virtual gko::size_type get_flops() const = 0;

    /**
     * Computes an estimate for the amount of memory accessed by the operation
     * (bytes).
     */
    virtual gko::size_type get_memory() const = 0;

    /**
     * Sets up all necessary data for a following call to
     * BenchmarkOperation::run.
     */
    virtual void prepare() {}

    /**
     * Computes the error between a reference solution and the solution provided
     * by this operation. The first value specifies whether the result is
     * structurally correct, the second value specifies the numerical error.
     */
    virtual std::pair<bool, double> validate() const = 0;

    /**
     * Executes the operation to be benchmarked.
     */
    virtual void run() = 0;


    /**
     * Allows the operation to write arbitrary information to the JSON output.
     */
    virtual void write_stats(rapidjson::Value& object,
                             rapidjson::MemoryPoolAllocator<>& allocator)
    {}
};


using Mtx = gko::matrix::Csr<etype, itype>;


std::unique_ptr<BenchmarkOperation> get_operation(std::string name,
                                                  const Mtx* matrix);

==> ginkgo-1.7.0.tar.gz: ginkgo-1.7.0/benchmark/sparse_blas/operations.hpp <==
/*******************************<GINKGO LICENSE>******************************
Copyright (c) 2017-2023, the Ginkgo authors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************<GINKGO LICENSE>*******************************/

#include <ginkgo/ginkgo.hpp>


#include <tuple>


#include "benchmark/utils/json.hpp"
#include "benchmark/utils/types.hpp"


class BenchmarkOperation {
public:
    virtual ~BenchmarkOperation() = default;

    /**
     * Computes an estimate for the number of FLOPs executed by the operation.
     */
    virtual gko::size_type get_flops() const = 0;

    /**
     * Computes an estimate for the amount of memory accessed by the operation
     * (bytes).
     */
    virtual gko::size_type get_memory() const = 0;

    /**
     * Sets up all necessary data for a following call to
     * BenchmarkOperation::run.
     */
    virtual void prepare() {}

    /**
     * Computes the error between a reference solution and the solution provided
     * by this operation. The first value specifies whether the result is
     * structurally correct, the second value specifies the numerical error.
     */
    virtual std::pair<bool, double> validate() const = 0;

    /**
     * Executes the operation to be benchmarked.
     */
    virtual void run() = 0;


    /**
     * Allows the operation to write arbitrary information to the JSON output.
     */
    virtual void write_stats(json& object) {}
};


using Mtx = gko::matrix::Csr<etype, itype>;


std::unique_ptr<BenchmarkOperation> get_operation(std::string name,
                                                  const Mtx* matrix);
